unit thrClean;

interface

{$define useabbrevia}

Uses Windows,Classes,Registry,SysUtils,CfgClean;

Type
  TMessageNotifyEvent = Procedure (Msg : String) of Object;

  TCleanThread = Class(TThread)
    FOnInfo,
    FOnError : TMessageNotifyEvent;
    FTotalFiles : Integer;
    FTotalTime : TDateTime;
    FVerbose : Boolean;
  private
    Procedure DoError(Msg : String);
    Procedure DoInfo(Msg : String);
    procedure InitDefaults(R: Tregistry; var DefExt: String; var DefSize: Integer);
    procedure InitDirData(R: TRegistry; const Entry: String; var Path, DirExt: String; var DirSize: Integer);
    procedure InitDirList(R: TRegistry; List: TStrings);
    procedure RecursivelyCleanDir(const Dir, Extensions: String; MinSize: Integer);
    procedure CleanDir(const Dir, Extensions: String; MinSize: Integer);
    procedure CleanFile(const Dir: String; const Info: TSearchRec);
    procedure CompressFile(FN, AFN: String);
  Public
    Procedure Execute; override;
    Property TotalFiles : Integer Read FTotalFiles;
    Property TotalTime : TDateTime Read FTotalTime;
    Property Verbose : Boolean Read FVerbose Write FVerbose;
    Property OnInfoMessage : TMessageNotifyEvent Read FOnInfo Write FOnInfo;
    Property OnErrorMessage : TMessageNotifyEvent Read FOnError Write FOnError;
  end;


ResourceString
  SErrNoPath = 'No path specified in entry %s';
  SErrTerminated = 'Cleaning process terminated. Cleared %d files in a total time %s';
  SErrUnexpected = 'An unexpected error occurred while cleaning the disks';
  SErrNoDir = 'No such directory: "%s"';
  SErrCannotDeleteArchiveFile = 'Could not delete old archive file: "%s"';

implementation

{$ifdef useabbrevia}
uses
  abzipper,abarctyp;

Const
   ArchiveExt = '.zip';

{$else}
uses zlib;

Const
  BufSize = 4096;
  ArchiveExt = '.gz';
{$endif}

{ TCleanThread }

procedure TCleanThread.InitDefaults(R : Tregistry; Var DefExt : String; Var DefSize : Integer);

begin
  DefSize:=DefaultSize;
  DefExt:=DefaultExtensions;
  With R Do
    If OpenKeyReadOnly(SkeyClean) then
      Try
      If ValueExists(SKeySize) then
        DefSize:=ReadInteger(SKeySize);
      if ValueExists(SkeyExtensions) then
        DefExt:=ReadString(SkeyExtensions);
      finally
        CloseKey;
      end;
end;

procedure TCleanThread.InitDirList(R : TRegistry; List : TStrings);

begin
  With R do
    If OpenKeyReadOnly(SKeyClean+SKeyDirs) then
      Try
        GetKeyNames(List);
      Finally
        CloseKey
      end;
end;

procedure TCleanThread.InitDirData(R : TRegistry; Const Entry : String;
                                   Var Path, DirExt : String;
                                   Var DirSize : Integer);

begin
  With R do
    If OpenKeyReadOnly(SKeyClean+SKeyDirs+'\'+Entry) then
      Try
        If ValueExists(SKeySize) then
          DirSize:=ReadInteger(SkeySize);
        If ValueExists(SKeyExtensions) then
          DirExt:=ReadString(SkeyExtensions);
        If ValueExists(SkeyPath) then
          Path:=ExcludeTrailingPathDelimiter(ReadString(SkeyPath));
      Finally
        CloseKey;
      end;
end;

procedure TCleanThread.Execute;

Var
  I,DefSize,DirSize : Integer;
  Path,DefExt,DirExt : String;
  L : TStrings;
  R : TRegistry;

begin
  Try
    R:=TRegistry.Create(KEY_READ);
    R.RootKey:=HKEY_LOCAL_MACHINE;
    Try
      InitDefaults(R,DefExt,DefSize);
      L:=TStringList.Create;
      Try
        InitDirList(R,L);
        DoInfo('Have L entries'+IntToStr(L.Count));
        I:=0;
        FTotalFiles:=0;
        FTotalTime:=Now;
        While (Not Terminated) and (I<L.Count) do
          begin
          DirSize:=DefSize;
          DirExt:=DefExt;
          Path:='';
          InitDirData(R,L[i],Path,DirExt,DirSize);
          If (Path<>'') then
            RecursivelyCleanDir(Path,DirExt,DirSize)
          else
            DoError(Format(SErrNoPath,[L[i]]));
          Inc(I);
          end;
        FTotalTime:=Now-FTotalTime;
        If Terminated then
          DoError(Format(SErrTerminated,[FTotalFiles,TimeToStr(FtotalTime)]));
      Finally
        L.Free;
      end;
    Finally
      R.Free;
    end;
  except
    On E : Exception do
      DoError(Format(SErrUnexpected,[E.Message]));
  end;
end;

procedure TCleanThread.RecursivelyCleanDir(Const Dir,Extensions : String; MinSize : Integer);

Var
  Info : TSearchRec;
  DirList : TStrings;
  I : Integer;

begin
  CleanDir(Dir,Extensions,MinSize);
  try
    DirList:=TStringList.Create;
    Try
      If FindFirst(Dir+'\*',faDirectory,Info)=0 then
        Try
          Repeat
            If ((Info.Attr and faDirectory)=faDirectory) and
               (Info.Name<>'.') and (info.Name<>'..') then
              DirList.Add(Info.Name)
          Until (FindNext(Info)<>0);
        finally
          FindClose(info);
        end;
      I:=0;
      While Not terminated and (I<Dirlist.Count) do
        begin
        RecursivelyCleanDir(Dir+'\'+Dirlist[i],Extensions,MinSize);
        inc(i);
        end;
    Finally
      DirList.Free;
    end;
  except
    On E : Exception do
      DoError(Format(SErrUnexpected,[E.Message]));
  end;
end;

procedure TCleanThread.CleanDir(Const Dir,Extensions : String; MinSize : Integer);

Var
  I : Integer;
  Ext,Lext : String;
  Info : TSearchRec;

begin
  DoInfo('Cleaning dir : '+Dir+' '+Extensions);
  If Not DirectoryExists(Dir) then
    begin
    DoError(Format(SErrNoDir,[Dir]));
    exit;
    end;
  Try
    LExt:=Extensions;
    While (Not Terminated) and (LExt<>'') do
      begin
      I:=Pos(';',Lext);
      If (I=0) then
        I:=Length(Lext)+1;
      Ext:=Copy(Lext,1,i-1);
      Delete(Lext,1,I);
      If FindFirst(Dir+'\*.'+Ext,0,Info)=0 then
        try
          repeat
            If (Info.Size div 1024)>MinSize then
              CleanFile(Dir,Info);
          Until Terminated or (FindNext(Info)<>0);
        Finally
          FindClose(Info);
        end;
      end;
  except
    On E : Exception do
      DoError(Format(SErrUnexpected,[E.Message]));
  end;
end;

{$ifdef UseAbbrevia}

Procedure TCleanThread.CompressFile(FN,AFN : String);

begin
  With TABZipper.Create(Nil) do
    Try
      StoreOptions:=[soStripPath];
      AutoSave:=true;
      FileName:=AFN;
      AddFiles(FN,0);
      // EVerything OK, Delete the old file.
      If Not DeleteFile(Fn) then
        RaiseLastOSError;
    Finally
      Free;
    end;
end;

{$else}
Procedure TCleanThread.CompressFile(FN,AFN : String);

Var
  InF,OutF : TFileStream;
  CS : TCompressionStream;
  Buf : Pointer;
  Count : Integer;

begin
  Inf:=TFileStream.Create(FN,fmOpenRead);
  Try
    OutF:=TFileStream.Create(AFN,fmCreate);
    Try
      GetMem(Buf,BufSize);
      Try
        CS:=TCompressionStream.Create(clMax,OutF);
        Try
          Repeat
            Count:=Inf.Read(Buf^,BufSize);
            If (Count>0) then
              CS.Write(Buf^,Count);
          Until (Count<BufSize);
        Finally
          CS.Free;
        end;
      Finally
        FreeMem(Buf);
      end;
    Finally
      OutF.Free;
    end;
  Finally
    Inf.Free;
  end;
  // EVerything OK, Delete the old file.
  If Not DeleteFile(FN) then
    RaiseLastOSError;
end;
{$endif}

procedure TCleanThread.CleanFile(Const Dir : String; Const Info : TSearchRec);


Var
  FN,AFN : String;
  FT,AFT : TDateTime;
  DoClean : Boolean;

begin
  Try
    FN:=IncludeTrailingPathDelimiter(Dir)+Info.Name;
    AFN:=FN+ArchiveExt;
    DoClean:=Not FileExists(AFN);
    If Not DoClean Then
      begin
      FT:=FileDateToDateTime(Info.Time);
      AFT:=FileDateToDateTime(FileAge(AFN));
      DoClean:=(AFT<FT);
      If DoClean then
        begin
        DoClean:=DeleteFile(AFN);
        If Not DoClean then
          DoError(Format(SErrCannotDeleteArchiveFile,[AFN]));
        end;
      end;
    If DoClean then
      begin
      // CleanService.PostInfo('Cleaning : '+FN);
      CompressFile(FN,AFN);
      Inc(FTotalFiles);
      end;
  Except
    On E : Exception do
      DoError(Format(SErrUnexpected,[E.Message]));
  end;
end;

procedure TCleanThread.DoError(Msg: String);
begin
  If Assigned(FOnError) then
    FOnError(Msg);
end;

procedure TCleanThread.DoInfo(Msg: String);
begin
  If FVerbose and Assigned(FOnInfo) then
    FOnInfo(Msg);
end;

end.
